"""This module provides functionality to modify a given Nuke script's sources
with given modify data read from a pickle file and convert all gizmos to
groups.
"""


# Import built-in modules
from __future__ import print_function

import os
import sys

PY3 = False
PY2 = False
if sys.version_info[0] == 3:
    PY3 = True
if sys.version_info[0] == 2:
    PY2 = True

if PY3:
    import pickle
elif PY2:
    import cPickle as pickle

# Import third-party modules
import nuke


# Add package root to sys.path because we need it for running standalone
this_dir = os.path.dirname(__file__)
smartcollect_root = os.path.join(this_dir, "..")
smartcollect_root = os.path.abspath(smartcollect_root)
if not smartcollect_root in sys.path:
    sys.path.insert(0, smartcollect_root)


def modify_nukescript(nukescript, data_path, source_path_mode,
                      recreate_source_path, gizmo_to_group):
    """Modify a given Nuke script's sources with given modify data read from
    given pickle file path.

    Args:
        nukescript (str): Nuke script to scan for footage.
        data_path (str): Absolute path of pickled data to use for modifying.
        source_path_mode (str): Switch to set the source files 'relative'
            or 'absolute'.
        recreate_source_path (str): If "True" recreate source paths so that we
            can re construct where the footage comes from.
        gizmo_to_group (str): If True convert all gizmos to groups.

    Returns:
        str: Absolute path of created pickled file.
    """

    relative_paths = (source_path_mode == "relative")

    print("@2:Modify nukescript source paths: {}".format(locals()))

    nuke.scriptOpen(nukescript)

    with open(data_path, "rb") as file_:
        modify_data = pickle.load(file_)

    # Set project directory to relative path
    if relative_paths:
        relative_project_root = "[python {nuke.script_directory()}]"
        nuke.root()["project_directory"].setValue(relative_project_root)

    for node_name, data in modify_data.items():

        node = nuke.toNode(node_name)
        fileknob = data["knob"]
        path = data["file"].replace("\\", "/")

        if node and node.knob(fileknob):

            # Relative paths.
            if relative_paths:
                print("@'1:data['file]: ", path)

                filename = os.path.basename(path)

                if recreate_source_path == "True":
                    root_name = os.path.dirname(path)
                    root_name = root_name.replace(nuke.script_directory(), "")
                    if root_name.startswith("/"):
                        root_name = root_name[1:]
                else:
                    root_name = os.path.dirname(path).split("/")[-1]
                    root_name = "footage/{}".format(root_name)

                path_update = "{}/{}".format(root_name, filename)
                path_update = os.path.normpath(path_update)

                print("@1:rootname: ", root_name)
                print("@1:filename: ", filename)

            # Absolute paths.
            else:
                path_update = path

            print("@2:Update {}.{} => {}".format(
                node_name, fileknob, path_update))

            node[fileknob].setValue(path_update.replace("\\", "/"))

    # Convert gizmos to group
    if gizmo_to_group == "True":
        print("@2: Converting gizmos to group")
        for node in nuke.allNodes():

            name = node.name()
            try:
                convert_to_group(node)
                print("@2: Converted %s" % name)

            except (RuntimeError, ValueError) as error:
                print("@2: Failed Converting %s %s" % (name, str(error)))
            except Exception:
                continue

    print("@2:Save script to: {}".format(nukescript))
    nuke.scriptSave("")
    nuke.scriptClose()


def convert_to_group(gizmo):
    """Convert gizmo to group by making group, renaming and re piping the
    inputs of the gizmo to the converted group.

    Args:
        gizmo (nuke.node): Gizmo node to convert to group.
    """
    # We need to select the gizmo, otherwise the node connection won't be set
    # correctly.
    gizmo.selectOnly()

    name = gizmo.name()
    inputs = [gizmo.input(index) for index in range(gizmo.inputs())]

    group = gizmo.makeGroup()

    group.setXYpos(int(gizmo.xpos()), int(gizmo.ypos()))

    for index in range(len(inputs)):
        group.setInput(index, inputs[index])

    nuke.delete(gizmo)
    group["name"].setValue(name)


try:
    modify_nukescript(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4],
                      sys.argv[5])
except IndexError:
    pass
