"""This module provides functionality to get a list of used sources in the
given Nuke script and writes the data as a pickled file to the temp area which
will then be processed by other functions."""


# Import built-in modules
import collections
import os
import sys

# Import third-party modules
import nuke

# Add package root to sys.path because we need it for running standalone
this_dir = os.path.dirname(__file__)
smartcollect_root = os.path.join(this_dir, "..")
smartcollect_root = os.path.abspath(smartcollect_root)
if not smartcollect_root in sys.path:
    sys.path.insert(0, smartcollect_root)

# Import local modules
from smartCollect import helper
from smartCollect.media import Media


# All nodes that have a 'file' knob and read input data using a single file.
# Exclusively 'Read' class because this class will be handled separately.
# Depending on the node the internal python name of the 'file' knob may change,
# that is why we store the 'file' knob name per node.
FILE_NODES_SIMPLE = {
    "OCIOCDLTransform": "file",
    "OCIOFileTransform": "file",
    "Vectorfield": "vfield_file",
    "Axis2": "file",
    "ReadGeo2": "file",
    "Light2": "file",
    "Camera2": "file",
    "DeepRead": "file",
    "AudioRead": "file",
    "BlinkScript": "kernelSourceFile",
    "Precomp": "file",
}

# All nodes that have a 'file' knob and read input data using a sequence.
# Exclusively 'Read' class because this class will be handled separately.
# Depending on the node the internal python name of the 'file' knob may change,
# that is why we store the 'file' knob name per node.
FILE_NODES_SEQUENCE = {
    "ScannedGrain": "fullGrain",
    # Since Nuke-11.2 SmartVector has no file knob anymore. We pre-check
    # if a file knob exists and continue early if it doesn't. In general we
    # do this pre-check with any node to be on a safe side here.
    "SmartVector": "file",
}


def get_all_reads():
    """Get all read nodes from current nuke script.

    Returns:
        list: List of all read nodes.

    """
    return [node.name() for node in nuke.allNodes("Read")]


def sanitize(path):
    """Sanitize the given path if it has a value.

    Args:
        path (str): The File path to sanitize.

    Returns:
        str: The sanitized path or empty string if it cannot be sanitized.

    """
    try:
        return path.replace("\\", "/")
    except AttributeError:
        return ""


def is_node_disabled(node):
    """Safely Check if the given node is disabled.

    Args:
        node (nuke.Node): The node to check.

    Returns:
        bool: True when the node is disabled, False otherwise.

    """
    return bool(node.knob("disable") and bool(node["disable"].value()))


def is_node_free_floating(node):
    """Check if node is free floating, i.e. not connected to anything.

    Args:
        node (nuke.Node): The node to check if it is free floating.

    Returns:
        bool: True when the given node is not connected to anything, False
            otherwise.

    """
    return not node.dependent()


def get_type(path):
    """Get the file type of the given path.

    Args:
        path: Path to get the file type from.

    Returns:
        The file type from the given path.

    """
    return os.path.splitext(path)[1].replace(".", "")


def get_nukescript_data(nukescript, signal=None, open_and_close=True):
    """Get all footage paths of all read nodes and additional file nodes in
    given nuke script. This will write the data as a pickled ordered
    dictionary and return its path which can then be parsed in other functions.

    Args:
        nukescript (str): Nuke script to scan for footage.

    Example:
        The generated pickle file is an ordered dictionary containing the
        following format:

        OrderedDict([
            ('Read1', {
                    'knob': 'file',
                    'file': '/path/to/checker.%04d.tif',
                    'filename': 'path/to/checker.0001.tif',
                    'node_name': 'Read1',
                    'node_class': 'Read',
                    'format': '3840x2160',
                    'first': 1001,
                    'last': 1020,
                    'disabled': True,
                }
            ),
            ('Read2', {
                    'knob': 'file',
                    'file': '/path/to/wheel.%04d.tif',
                    'filename': 'path/to/wheel.0001.tif',
                    'node_name': 'Read2',
                    'node_class': 'Read',
                    'format': '1920x1080',
                    'first': 1001,
                    'last': 1050,
                    'disabled': False,
                 }
            ),
            ...
        ])

    Returns:
        str: Absolute path of created pickled file.

    """
    if open_and_close:
        nuke.scriptOpen(nukescript)

    data = collections.OrderedDict()
    files_paths = []

    for node in nuke.allNodes(recurseGroups=True):

        node_name = node.fullName()
        node_class = node.Class()

        file_value = ""

        # Process all Read nodes.
        if node_class == "Read":

            file_value = sanitize(node["file"].value())
            if file_value in files_paths:
                continue

            media = Media(file_value)

            data[node_name] = {
                "archive": True,
                "filename": nuke.filename(node, nuke.REPLACE),
                "file": file_value,
                "knob": "file",
                "node_name": node_name,
                "node_class": node_class,
                "format": "{0}x{1}".format(node.width(), node.height()),
                "first": node["first"].value(),
                "last": node["last"].value(),
                "disabled": is_node_disabled(node),
                "free_floating": is_node_free_floating(node),
                "size": media.get_size(),
                "ext": get_type(file_value),
            }

        # Process single footage files.
        elif node_class in FILE_NODES_SIMPLE.keys():
            file_knob = node.knob(FILE_NODES_SIMPLE[node_class])
            if not file_knob:
                continue

            file_value = sanitize(file_knob.getValue())
            if file_value in files_paths:
                continue

            media = Media(file_value)

            if file_value:
                data[node_name] = {
                    "archive": True,
                    "filename": file_value,
                    "file": file_value,
                    "knob": FILE_NODES_SIMPLE[node_class],
                    "disabled": is_node_disabled(node),
                    "free_floating": is_node_free_floating(node),
                    "size": media.get_size(),
                    "ext": get_type(file_value),
                }

        # Process sequence footage files. Read out sequence by inserting the
        # file value in tmp read node and then using nuke.filename().
        elif node_class in FILE_NODES_SEQUENCE:
            file_knob = node.knob(FILE_NODES_SEQUENCE[node_class])
            if not file_knob:
                continue

            file_value = sanitize(file_knob.getValue())
            if file_value in files_paths:
                continue

            read_tmp = nuke.nodes.Read()
            read_tmp["file"].setValue(file_value)
            filename = nuke.filename(read_tmp, nuke.REPLACE)
            media = Media(file_value)

            data[node_name] = {
                "archive": True,
                "filename": filename,
                "file": file_value,
                "knob": "file",
                "disabled": is_node_disabled(node),
                "free_floating": is_node_free_floating(node),
                "size": media.get_size(),
                "ext": get_type(file_value),
            }

            nuke.delete(read_tmp)

        # Collect anything else that has a file knob.
        elif node.knob("file") and node["file"].value():
            file_value = sanitize(node["file"].value())
            media = Media(file_value)

            data[node_name] = {
                "archive": True,
                "node_name": node_name,
                "node_class": node_class,
                "filename": file_value,
                "file": file_value,
                "knob": "file",
                "disabled": is_node_disabled(node),
                "free_floating": is_node_free_floating(node),
                "size": media.get_size(),
                "ext": get_type(file_value),
            }

            if file_value in files_paths:
                continue

        files_paths.append(file_value)

    if open_and_close:
        # Write data to pickle file.
        pickle_data_path = helper.write_pickle_data(data)
        # Stdout pickle_data path.
        print("@data", pickle_data_path)
        nuke.scriptClose()

    else:
        return data


if __name__ == "__main__":
    get_nukescript_data(sys.argv[1])
